@@ -38,15 +38,11 @@ trap(void);
 /* To solve type punned error */
 #define VOID(ptr) ((void **)((void *)ptr))
 
-#include "logging.h"
-#include "glusterfs.h"
-#include "locking.h"
-#include "mem-pool.h"
-#include "compat-uuid.h"
-#include "iatt.h"
-#include "libglusterfs-messages.h"
+#include "glusterfs/mem-pool.h"
+#include "glusterfs/compat-uuid.h"
+#include "glusterfs/iatt.h"
+#include "glusterfs/libglusterfs-messages.h"
 #include "protocol-common.h"
-#include "iobuf.h"
 
 #define STRINGIFY(val) #val
 #define TOSTRING(val) STRINGIFY(val)
@@ -61,8 +57,8 @@ trap(void);
 
 #define min(a, b) ((a) < (b) ? (a) : (b))
 #define max(a, b) ((a) > (b) ? (a) : (b))
-#define roof(a, b) ((((a) + (b)-1) / ((b != 0) ? (b) : 1)) * (b))
-#define floor(a, b) (((a) / ((b != 0) ? (b) : 1)) * (b))
+#define gf_roof(a, b) ((((a) + (b)-1) / ((b != 0) ? (b) : 1)) * (b))
+#define gf_floor(a, b) (((a) / ((b != 0) ? (b) : 1)) * (b))
 
 #define IPv4_ADDR_SIZE 32
 
@@ -147,14 +143,8 @@ trap(void);
 /* pthread related */
 /* as per the man page, thread-name should be at max 16 bytes */
 /* with prefix of 'glfs_' (5), we are left with 11 more bytes */
-#define GF_THREAD_NAMEMAX 11
+#define GF_THREAD_NAME_LIMIT 16
 #define GF_THREAD_NAME_PREFIX "glfs_"
-#define GF_THREAD_NAME_PREFIX_LEN 5
-
-#include <stdbool.h>
-#define gf_boolean_t bool
-#define _gf_false false
-#define _gf_true true
 
 /*
  * we could have initialized these as +ve values and treated
@@ -174,7 +164,9 @@ enum _gf_special_pid {
     GF_CLIENT_PID_BITD = -8,
     GF_CLIENT_PID_SCRUB = -9,
     GF_CLIENT_PID_TIER_DEFRAG = -10,
-    GF_SERVER_PID_TRASH = -11
+    GF_SERVER_PID_TRASH = -11,
+    GF_CLIENT_PID_ADD_REPLICA_MOUNT = -12,
+    GF_CLIENT_PID_SET_UTIME = -13,
 };
 
 enum _gf_xlator_ipc_targets {
@@ -447,7 +439,7 @@ BIT_VALUE(unsigned char *array, unsigned int index)
     } while (0)
 #endif
 
-#define GF_ABORT(msg)                                                          \
+#define GF_ABORT(msg...)                                                       \
     do {                                                                       \
         gf_msg_callingfn("", GF_LOG_CRITICAL, 0, LG_MSG_ASSERTION_FAILED,      \
                          "Assertion failed: " msg);                            \
@@ -517,65 +509,193 @@ static inline struct iovec *
 iov_dup(const struct iovec *vector, int count)
 {
     int bytecount = 0;
-    int i;
     struct iovec *newvec = NULL;
 
     bytecount = (count * sizeof(struct iovec));
     newvec = GF_MALLOC(bytecount, gf_common_mt_iovec);
-    if (!newvec)
-        return NULL;
-
-    for (i = 0; i < count; i++) {
-        newvec[i].iov_len = vector[i].iov_len;
-        newvec[i].iov_base = vector[i].iov_base;
+    if (newvec != NULL) {
+        memcpy(newvec, vector, bytecount);
     }
 
     return newvec;
 }
 
-static inline int
-iov_subset(struct iovec *orig, int orig_count, off_t src_offset,
-           off_t dst_offset, struct iovec *new)
+typedef struct _iov_iter {
+    const struct iovec *iovec;
+    void *ptr;
+    uint32_t len;
+    uint32_t count;
+} iov_iter_t;
+
+static inline bool
+iov_iter_init(iov_iter_t *iter, const struct iovec *iovec, uint32_t count,
+              uint32_t offset)
 {
-    int new_count = 0;
-    int i;
-    off_t offset = 0;
-    size_t start_offset = 0;
-    size_t end_offset = 0, origin_iov_len = 0;
+    uint32_t len;
 
-    for (i = 0; i < orig_count; i++) {
-        origin_iov_len = orig[i].iov_len;
+    while (count > 0) {
+        count--;
+        len = iovec->iov_len;
+        if (offset < len) {
+            iter->ptr = iovec->iov_base + offset;
+            iter->len = len - offset;
+            iter->iovec = iovec + 1;
+            iter->count = count;
 
-        if ((offset + orig[i].iov_len < src_offset) || (offset > dst_offset)) {
-            goto not_subset;
+            return true;
         }
-
-        if (!new) {
-            goto count_only;
-        }
-
-        start_offset = 0;
-        end_offset = orig[i].iov_len;
-
-        if (src_offset >= offset) {
-            start_offset = (src_offset - offset);
-        }
-
-        if (dst_offset <= (offset + orig[i].iov_len)) {
-            end_offset = (dst_offset - offset);
-        }
-
-        new[new_count].iov_base = orig[i].iov_base + start_offset;
-        new[new_count].iov_len = end_offset - start_offset;
-
-    count_only:
-        new_count++;
-
-    not_subset:
-        offset += origin_iov_len;
+        offset -= len;
     }
 
-    return new_count;
+    memset(iter, 0, sizeof(*iter));
+
+    return false;
+}
+
+static inline bool
+iov_iter_end(iov_iter_t *iter)
+{
+    return iter->count == 0;
+}
+
+static inline bool
+iov_iter_next(iov_iter_t *iter, uint32_t size)
+{
+    GF_ASSERT(size <= iter->len);
+
+    if (iter->len > size) {
+        iter->len -= size;
+        iter->ptr += size;
+
+        return true;
+    }
+    if (iter->count > 0) {
+        iter->count--;
+        iter->ptr = iter->iovec->iov_base;
+        iter->len = iter->iovec->iov_len;
+        iter->iovec++;
+
+        return true;
+    }
+
+    memset(iter, 0, sizeof(*iter));
+
+    return false;
+}
+
+static inline uint32_t
+iov_iter_copy(iov_iter_t *dst, iov_iter_t *src, uint32_t size)
+{
+    uint32_t len;
+
+    len = src->len;
+    if (len > dst->len) {
+        len = dst->len;
+    }
+    if (len > size) {
+        len = size;
+    }
+    memcpy(dst->ptr, src->ptr, len);
+
+    return len;
+}
+
+static inline uint32_t
+iov_iter_to_iovec(iov_iter_t *iter, struct iovec *iovec, int32_t idx,
+                  uint32_t size)
+{
+    uint32_t len;
+
+    len = iter->len;
+    if (len > size) {
+        len = size;
+    }
+    iovec[idx].iov_base = iter->ptr;
+    iovec[idx].iov_len = len;
+
+    return len;
+}
+
+static inline int
+iov_subset(struct iovec *src, int src_count, uint32_t start, uint32_t size,
+           struct iovec **dst, int32_t dst_count)
+{
+    struct iovec iovec[src_count];
+    iov_iter_t iter;
+    uint32_t len;
+    int32_t idx;
+
+    if ((size == 0) || !iov_iter_init(&iter, src, src_count, start)) {
+        return 0;
+    }
+
+    idx = 0;
+    do {
+        len = iov_iter_to_iovec(&iter, iovec, idx, size);
+        idx++;
+        size -= len;
+    } while ((size > 0) && iov_iter_next(&iter, len));
+
+    if (*dst == NULL) {
+        *dst = iov_dup(iovec, idx);
+        if (*dst == NULL) {
+            return -1;
+        }
+    } else if (idx > dst_count) {
+        return -1;
+    } else {
+        memcpy(*dst, iovec, idx * sizeof(struct iovec));
+    }
+
+    return idx;
+}
+
+static inline int
+iov_skip(struct iovec *iovec, uint32_t count, uint32_t size)
+{
+    uint32_t len, idx;
+
+    idx = 0;
+    while ((size > 0) && (idx < count)) {
+        len = iovec[idx].iov_len;
+        if (len > size) {
+            iovec[idx].iov_len -= size;
+            iovec[idx].iov_base += size;
+            break;
+        }
+        idx++;
+        size -= len;
+    }
+
+    if (idx > 0) {
+        memmove(iovec, iovec + idx, (count - idx) * sizeof(struct iovec));
+    }
+
+    return count - idx;
+}
+
+static inline size_t
+iov_range_copy(const struct iovec *dst, uint32_t dst_count, uint32_t dst_offset,
+               const struct iovec *src, uint32_t src_count, uint32_t src_offset,
+               uint32_t size)
+{
+    iov_iter_t src_iter, dst_iter;
+    uint32_t len, total;
+
+    if ((size == 0) || !iov_iter_init(&src_iter, src, src_count, src_offset) ||
+        !iov_iter_init(&dst_iter, dst, dst_count, dst_offset)) {
+        return 0;
+    }
+
+    total = 0;
+    do {
+        len = iov_iter_copy(&dst_iter, &src_iter, size);
+        total += len;
+        size -= len;
+    } while ((size > 0) && iov_iter_next(&src_iter, len) &&
+             iov_iter_next(&dst_iter, len));
+
+    return total;
 }
 
 static inline void
@@ -614,54 +734,38 @@ iov_load(const struct iovec *vector, int count, char *buf, int size)
 static inline size_t
 iov_copy(const struct iovec *dst, int dcnt, const struct iovec *src, int scnt)
 {
-    size_t ret = 0;
-    size_t left = 0;
-    size_t min_i = 0;
-    int s_i = 0, s_ii = 0;
-    int d_i = 0, d_ii = 0;
+    return iov_range_copy(dst, dcnt, 0, src, scnt, 0, UINT32_MAX);
+}
 
-    ret = min(iov_length(dst, dcnt), iov_length(src, scnt));
-    left = ret;
+/* based on the amusing discussion @ https://rusty.ozlabs.org/?p=560 */
+static bool
+memeqzero(const void *data, size_t length)
+{
+    const unsigned char *p = data;
+    size_t len;
 
-    while (left) {
-        min_i = min(dst[d_i].iov_len - d_ii, src[s_i].iov_len - s_ii);
-        memcpy(dst[d_i].iov_base + d_ii, src[s_i].iov_base + s_ii, min_i);
-
-        d_ii += min_i;
-        if (d_ii == dst[d_i].iov_len) {
-            d_ii = 0;
-            d_i++;
-        }
-
-        s_ii += min_i;
-        if (s_ii == src[s_i].iov_len) {
-            s_ii = 0;
-            s_i++;
-        }
-
-        left -= min_i;
+    /* Check first 16 bytes manually */
+    for (len = 0; len < 16; len++) {
+        if (!length)
+            return true;
+        if (*p)
+            return false;
+        p++;
+        length--;
     }
 
-    return ret;
+    /* Now we know that's zero, memcmp with self. */
+    return memcmp(data, p, length) == 0;
 }
 
 static inline int
 mem_0filled(const char *buf, size_t size)
 {
-    int i = 0;
-    int ret = 0;
-
-    for (i = 0; i < size; i++) {
-        ret = buf[i];
-        if (ret)
-            break;
-    }
-
-    return ret;
+    return !memeqzero(buf, size);
 }
 
 static inline int
-iov_0filled(struct iovec *vector, int count)
+iov_0filled(const struct iovec *vector, int count)
 {
     int i = 0;
     int ret = 0;
@@ -675,19 +779,6 @@ iov_0filled(struct iovec *vector, int count)
     return ret;
 }
 
-static inline void *
-memdup(const void *ptr, size_t size)
-{
-    void *newptr = NULL;
-
-    newptr = GF_MALLOC(size, gf_common_mt_memdup);
-    if (!newptr)
-        return NULL;
-
-    memcpy(newptr, ptr, size);
-    return newptr;
-}
-
 typedef enum {
     gf_timefmt_default = 0,
     gf_timefmt_FT = 0, /* YYYY-MM-DD hh:mm:ss */
@@ -806,10 +897,6 @@ gf_string2uint32_base10(const char *str, uint32_t *n);
 int
 gf_string2uint64_base10(const char *str, uint64_t *n);
 int
-gf_string2bytesize(const char *str, uint64_t *n);
-int
-gf_string2bytesize_size(const char *str, size_t *n);
-int
 gf_string2bytesize_uint64(const char *str, uint64_t *n);
 int
 gf_string2bytesize_int64(const char *str, int64_t *n);
@@ -830,12 +917,12 @@ int
 gf_unlockfd(int fd);
 
 int
-get_checksum_for_file(int fd, uint32_t *checksum);
+get_checksum_for_file(int fd, uint32_t *checksum, int op_version);
 int
 log_base2(unsigned long x);
 
 int
-get_checksum_for_path(char *path, uint32_t *checksum);
+get_checksum_for_path(char *path, uint32_t *checksum, int op_version);
 int
 get_file_mtime(const char *path, time_t *stamp);
 char *
@@ -871,7 +958,8 @@ valid_ipv4_address(char *address, int length, gf_boolean_t wildcard_acc);
 char
 valid_ipv6_address(char *address, int length, gf_boolean_t wildcard_acc);
 char
-valid_internet_address(char *address, gf_boolean_t wildcard_acc);
+valid_internet_address(char *address, gf_boolean_t wildcard_acc,
+                       gf_boolean_t cidr);
 gf_boolean_t
 valid_mount_auth_address(char *address);
 gf_boolean_t
@@ -945,10 +1033,24 @@ gf_set_timestamp(const char *src, const char *dest);
 
 int
 gf_thread_create(pthread_t *thread, const pthread_attr_t *attr,
-                 void *(*start_routine)(void *), void *arg, const char *name);
+                 void *(*start_routine)(void *), void *arg, const char *name,
+                 ...) __attribute__((__format__(__printf__, 5, 6)));
+
+int
+gf_thread_vcreate(pthread_t *thread, const pthread_attr_t *attr,
+                  void *(*start_routine)(void *), void *arg, const char *name,
+                  va_list args);
 int
 gf_thread_create_detached(pthread_t *thread, void *(*start_routine)(void *),
-                          void *arg, const char *name);
+                          void *arg, const char *name, ...)
+    __attribute__((__format__(__printf__, 4, 5)));
+
+void
+gf_thread_set_name(pthread_t thread, const char *name, ...)
+    __attribute__((__format__(__printf__, 2, 3)));
+
+void
+gf_thread_set_vname(pthread_t thread, const char *name, va_list args);
 gf_boolean_t
 gf_is_pid_running(int pid);
 gf_boolean_t
@@ -1059,4 +1161,10 @@ gf_replace_old_iatt_in_dict(struct _dict *);
 int
 gf_replace_new_iatt_in_dict(struct _dict *);
 
+xlator_cmdline_option_t *
+find_xlator_option_in_cmd_args_t(const char *option_name, cmd_args_t *args);
+
+int
+gf_d_type_from_ia_type(ia_type_t type);
+
 #endif /* _COMMON_UTILS_H */
